;Delay and Mute 14 PCB
;last modified 2022-09-10

; When relay is ON the audio is NOT muted.
; Change SYMBOL "Start_Delay" to set startup delay time in milliseconds

;Picaxe 08M2 Pinout
;Pin1 = +V
;Pin2 = C.5 (In, Serial In)
;Pin3 = C.4 (Touch/ADC/Out/In)
;Pin4 = C.3 (In)
;Pin5 = C.2 (In/Out/ADC/pwm/tune/SRQ/hi2c sda)
;Pin6 = C.1 (In/Out/ADC/hserin/SRI/hi2c scl)
;Pin7 = C.0 /Serial Out/Out//hserout/DAC)
;Pin8 = 0V

#Picaxe 08M2
#No_Data
SETFREQ m4

;allow time for system to stabilize
PAUSE 100

'Make all I/O pins inputs initially 
Input C.4 
Input C.2
Input C.1

;Pin assignments
;Control Inputs
SYMBOL Read_Level = C.2 '0=muted 1=unmuted
;Audio Output (Relay) Control
SYMBOL Rel_CtrlA = C.4
SYMBOL Rel_CtrlB = C.1

;variables storage

SYMBOL Current_Level = b2 'instantaneously read level
SYMBOL Mute_Level  = b3 'degraded supply level that causes output to be muted
SYMBOL Normal_Level = b4 'normal supply level
SYMBOL Start_Delay = W4
SYMBOL Delay_Calib = W5
Delay_Calib = 1

;*******************************************************************************
;*******************************************************************************
;Startup Delay in milliseconds to avoid turn-on thumps (e.g. 4000 = 4 seconds) *
Start_Delay = 4000 '                                                           *
;Note: actual delay will be very slightly longer due to program execution time *
;*******************************************************************************
;*******************************************************************************

;Pause the specified delay time
PAUSE Start_Delay

;INITIALIZE THE BOARD

;do not proceed if the circuit is manually muted (Current_Level < 20)
'get current level reading
READADC Read_Level, Current_Level 
IF Current_Level < 20 THEN 'then Mute switch is ON
	GOSUB Rel_Ctrl_OFF
	DO WHILE Current_Level < 20 'stay here until the mute switch is unmuted
		READADC Read_Level, Current_Level
	LOOP
	PAUSE 1000 'allow voltages to return to normal
ENDIF

'determine normal and sagging (75% of normal) mute trigger levels 
'get current nominal voltage level reading
READADC Read_Level, Current_Level
Normal_Level = Current_Level
'set Mute Level to 75% of Normal Level
Mute_Level = Normal_Level*75/100
'send data to terminal for troubleshooting

Sertxd ("Startup Normal and Mute (75% Normal) Levels are ",#Normal_Level," & ",#Mute_Level,13,10)
Sertxd ("Programmed Start Delay is ",#Start_Delay," milliseconds",13,10)
;Turn relay ON (unmute)
GOSUB Rel_Ctrl_ON 

'main program is a continuous loop to watch for Power Level changes.
Main:

'reset Normal and Mute Levels @ 10 seconds after power supply has settled
IF Delay_Calib < 1000 THEN
	Delay_Calib = Delay_Calib + 1
	IF Delay_Calib > 999 THEN
		'determine normal and sagging (75% of normal) mute trigger levels 
		'get current nominal voltage level reading
		READADC Read_Level, Current_Level
		Normal_Level = Current_Level
		'set Mute Level to 75% of Normal Level
		Mute_Level = Normal_Level*75/100
		'send data to terminal for troubleshooting
		Sertxd ("DELAYED Normal and Mute (75% Normal) Levels are ",#Normal_Level," & ",#Mute_Level,13,10)
		PAUSE 5000
	ENDIF
ENDIF	

'get current level reading
READADC Read_Level, Current_Level
'compare current power level to disconnect level and act accordingly
IF Current_Level < Mute_Level THEN
	GOSUB Mute_Audio
ENDIF

'Loop back to Main
GOTO Main

'if current level is < Mute_Level then mute the audio
MUTE_AUDIO:
GOSUB Rel_Ctrl_OFF
DO WHILE Current_Level < Mute_Level 'remain here until Current Level rises above Mute Level
	READADC Read_Level, Current_Level
LOOP
GOSUB Rel_Ctrl_ON:
RETURN

	
'Relay switching subroutines 
'Always make both pins "input" before changing their output state to avoid conflicting states

'Output disable
Rel_Ctrl_OFF:
'troubleshooting terminal data
Sertxd ("Output Muted",13,10)
Input Rel_CtrlA
Input Rel_CtrlB
PAUSE 10
Low Rel_CtrlA
Low Rel_CtrlB
PAUSE 75
RETURN

'Output enable
Rel_Ctrl_ON:
'troubleshooting terminal data
Sertxd ("Output Unmuted",13,10)
Input Rel_CtrlA
Input Rel_CtrlB
PAUSE 10
High Rel_CtrlA
High Rel_CtrlB
PAUSE 75
RETURN	